﻿/*
*
*  JSON datové typy
*
*/

-- ===================
-- Kontrola datového typu JSON
-- ===================

select 
'{"hmotnost":[
	{"nazev":"kilogram","zkr":"kg","koeficient":1},
	{"nazev":"gram","zkr":"gr","koeficient":0.001},
	{"nazev":"mikrogram","zkr":"μg","koeficient":0.000000001},
	{"nazev":"tuna","zkr":"t","koeficient":1000}]}'::json;

-- ============================================
-- Tabulka obsahuje v jednom sloupci různá JSON
-- ============================================

-- Vytvoření tabulky jednotka

drop table if exists jednotka;
create table jednotka
(id serial primary key,
 typ varchar(80) not null unique,
 hodnota json not null);


-- Vložení dat
-- První záznam jsou jednotky pro hmotnost
insert into jednotka
(typ,hodnota)
values
(
'hmotnost',
'{"hmotnost":[
	{"nazev":"kilogram","zkr":"kg","koeficient":1},
	{"nazev":"gram","zkr":"gr","koeficient":0.001},
	{"nazev":"mikrogram","zkr":"μg","koeficient":0.000000001},
	{"nazev":"tuna","zkr":"t","koeficient":1000}]}'::json
 ); 

-- druhý záznam jsou jednotky pro čas
insert into jednotka
(typ,hodnota)
 values
 (
 'cas',
 '{"cas":[
	{"nazev":"sekunda","zkr":"s","koeficient":1},
	{"nazev":"minuta","zkr":"min","koeficient":60},
	{"nazev":"den (střední sluneční)","zkr":"d","koeficient":86400},
	{"nazev":"sol (střední Mars)","zkr":"sol","koeficient":88775}
	]}':: json
  );

-- delete from jednotka;


-- Výběr času
-- Vyber ciselniku casu

-- Vytvoření kompositního typu podle kterého se budou publikovat hodnoty
create type t_jednotka as (nazev varchar(80) ,zkr varchar(10), koeficient numeric);

-- Vlastní dotaz pro zobrazení hodnot
select d.*
from
(select json_array_elements(json_extract_path(hodnota,'cas')) as jednotka
  from jednotka
where typ = 'cas') j
,json_populate_record(null::t_jednotka,j.jednotka) d;

/*
         nazev          | zkr | koeficient
------------------------+-----+------------
 sekunda                | s   |          1
 minuta                 | min |         60
 den (střední sluneční) | d   |      86400
 sol (střední Mars)     | sol |      88775
(4 řádky)
*/

-- Vysvětlení: část dotazu ktera extrahuje hodnotu prvku z cesty

select json_extract_path(hodnota,'cas') as hodnota
from jednotka
where typ = 'cas';

/*
                                  hodnota
----------------------------------------------------------------------------
 [\r                                                                       +
         {"nazev":"sekunda","zkr":"s","koeficient":1},\r                   +
         {"nazev":"minuta","zkr":"min","koeficient":60},\r                 +
         {"nazev":"den (střední sluneční)","zkr":"d","koeficient":86400},\r+
         {"nazev":"sol (střední Mars)","zkr":"sol","koeficient":88775}\r   +
         ]
(1 řádka)
*/


-- Vysvětlení : část dotazu, která z pole hodnot extrahuje hodnoty
select json_array_elements(json_extract_path(hodnota,'cas')) as jednotka
  from jednotka
where typ = 'cas'

/*
                            jednotka
-----------------------------------------------------------------
 {"nazev":"sekunda","zkr":"s","koeficient":1}
 {"nazev":"minuta","zkr":"min","koeficient":60}
 {"nazev":"den (střední sluneční)","zkr":"d","koeficient":86400}
 {"nazev":"sol (střední Mars)","zkr":"sol","koeficient":88775}
(4 řádky)
*/


-- Výběr hmotnosti

select d.*
from
(select json_array_elements(json_extract_path(hodnota,'hmotnost')) as jednotka
  from jednotka
where typ = 'hmotnost') j
,json_populate_record(null::t_jednotka,j.jednotka) d;

/*
"kilogram";"kg";1
"gram";"gr";0.001
"mikrogram";"μg";0.000000001
"tuna";"t";1000
*/


-- ==================================================
-- Tabulka obsahuje v jednom sloupci stejne JSON 
-- ==================================================

/*
Příklad obsahuje příklad meteo data předpovědí 
*/

drop table if exists weather_day

-- Vytvoreni tabulky
create table weather_day
(id serial primary key,
 place varchar(80) not null,
 meteo_date date not null,
 meteo_vals json not null,
 unique(place,meteo_date));

-- Vložení dat za Vevěří a Prahu 21.3.2016
insert into weather_day
(place,meteo_date,meteo_vals)
values
('Veveri',
to_date('21.3.2016','DD.MM.YYYY'),
'{"meteo":[
{"time":9,"temp":6,"feels":5,"humidity":61},
{"time":10,"temp":7,"feels":5,"humidity":55},
{"time":11,"temp":8,"feels":5,"humidity":52},
{"time":12,"temp":8,"feels":5,"humidity":53},
{"time":13,"temp":8,"feels":6,"humidity":52},
{"time":14,"temp":9,"feels":6,"humidity":52},
{"time":15,"temp":9,"feels":6,"humidity":52},
{"time":16,"temp":8,"feels":6,"humidity":55}]}'::JSON),
('Prague',
to_date('21.3.2016','DD.MM.YYYY'),
'{"meteo":[
{"time":9,"temp":5,"feels":2,"humidity":73},
{"time":10,"temp":6,"feels":3,"humidity":69},
{"time":11,"temp":7,"feels":4,"humidity":65},
{"time":12,"temp":7,"feels":5,"humidity":61},
{"time":13,"temp":8,"feels":6,"humidity":59},
{"time":14,"temp":8,"feels":6,"humidity":59},
{"time":15,"temp":8,"feels":6,"humidity":58},
{"time":16,"temp":8,"feels":6,"humidity":59}]}'::JSON);

 
-- Výběr dat z celé tabulky

create type t_meteo as (time smallint,temp smallint,feels smallint,humidity smallint);

select w.id,w.place,w.meteo_date,d.*
from
(select json_array_elements(json_extract_path(meteo_vals,'meteo')) as meteo_val,id,place,meteo_date
  from weather_day) w
,json_populate_record(null::t_meteo,w.meteo_val) d;


/*
 id | place  | meteo_date | time | temp | feels | humidity
----+--------+------------+------+------+-------+----------
  1 | Veveri | 2016-03-21 |    9 |    6 |     5 |       61
  1 | Veveri | 2016-03-21 |   10 |    7 |     5 |       55
  1 | Veveri | 2016-03-21 |   11 |    8 |     5 |       52
  1 | Veveri | 2016-03-21 |   12 |    8 |     5 |       53
  1 | Veveri | 2016-03-21 |   13 |    8 |     6 |       52
  1 | Veveri | 2016-03-21 |   14 |    9 |     6 |       52
  1 | Veveri | 2016-03-21 |   15 |    9 |     6 |       52
  1 | Veveri | 2016-03-21 |   16 |    8 |     6 |       55
  2 | Prague | 2016-03-21 |    9 |    5 |     2 |       73
  2 | Prague | 2016-03-21 |   10 |    6 |     3 |       69
  2 | Prague | 2016-03-21 |   11 |    7 |     4 |       65
  2 | Prague | 2016-03-21 |   12 |    7 |     5 |       61
  2 | Prague | 2016-03-21 |   13 |    8 |     6 |       59
  2 | Prague | 2016-03-21 |   14 |    8 |     6 |       59
  2 | Prague | 2016-03-21 |   15 |    8 |     6 |       58
  2 | Prague | 2016-03-21 |   16 |    8 |     6 |       59
(16 řádek)
*/  

-- Zrušení tabulky
 drop table weather_day;
 drop type t_meteo;