﻿/*
*     Cizí klíč II - pokročileší funkce
*
*
*/

-- ======================================================
-- Proč cizí klíč není výhodné použít jako primarní klíč
-- ======================================================

drop table if exists jazyk cascade;
drop table if exists kniha cascade;

-- Vytvoření číselníku jazyk 
create table jazyk
(jazyk varchar(60) primary key);

insert into jazyk
values
('Čeština'),
('Francouzština'),
('Nemčina');


-- Vytvoření tabulky kniha
create table kniha
(id serial primary key,
 nazev varchar(80) not null unique,
 jazyk varchar(60) references jazyk);

 Insert into kniha
 (nazev,jazyk)
 values
 ('Tajuplný ostrov','Čeština'),
 ('île mystérieuse','Francouzština'),
 ('geheimnisvolle Insel','Nemčina');

-- Zobrazení chybové hlášky protože hodnota je použita v jiné tabulce.
update jazyk
set jazyk = 'Němčina'
where jazyk = 'Nemčina';

/*
ERROR:  update or delete on table "jazyk" violates foreign key constraint "kniha_jazyk_fkey" on table "kniha"
DETAIL:  Key (jazyk)=(Nemčina) is still referenced from table "kniha".
********** Chyba **********

ERROR: update or delete on table "jazyk" violates foreign key constraint "kniha_jazyk_fkey" on table "kniha"
Stav SQL: 23503
Podrobnosti:Key (jazyk)=(Nemčina) is still referenced from table "kniha".
*/


 -- Jak to obejit
 -- Nastavení modifikace existujícícho klíče

-- Zruší se existující cizí klíč
ALTER TABLE public.kniha DROP CONSTRAINT kniha_jazyk_fkey;

-- Přidá se nový klíč u kterého se modifikuje akce na update na CASCADE z NO ACTION
 ALTER TABLE public.kniha
  ADD CONSTRAINT kniha_jazyk_fkey FOREIGN KEY (jazyk)
      REFERENCES public.jazyk (jazyk) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE NO ACTION;

-- Znovu provedeme update
update jazyk
set jazyk = 'Němčina'
where jazyk = 'Nemčina';

-- Výsledek

select * from kniha;

/*
1;"Tajuplný ostrov";"Čeština"
2;"île mystérieuse";"Francouzština"
3;"geheimnisvolle Insel";"Němčina"
*/

-- Přesto doporučuji nepoužívat ! Čas na update.

-- ===============================================
-- Fatální následky při použití on delete cascade
-- ===============================================

-- Zruší se existující cizí klíč
ALTER TABLE public.kniha DROP CONSTRAINT kniha_jazyk_fkey;

-- Přidá se nový klíč u kterého se modifikuje akce na delete na CASCADE z NO ACTION
 ALTER TABLE public.kniha
  ADD CONSTRAINT kniha_jazyk_fkey FOREIGN KEY (jazyk)
      REFERENCES public.jazyk (jazyk) MATCH SIMPLE
      ON UPDATE CASCADE ON DELETE CASCADE;

-- Znovu provedeme update
delete from jazyk
where jazyk = 'Němčina';

-- Výsledek
select * from kniha; 

/*
1;"Tajuplný ostrov";"Čeština"
2;"île mystérieuse";"Francouzština"
*/     

 -- Doporučení :
 -- Používat výhradně v spojení tabulek master-slave. Nikdy nevázat na číselníky.


 -- =========================================
 -- Další možnosti akce na update nebo delete
 -- =========================================

-- =========
-- 1. NULL 1
-- =========
 
-- Případ, při vkládaní záznamu použijeme null hodnotu. (Musí být definováno v tabulce.)
-- V případě, že se nazná hodnota

insert into kniha
(nazev,jazyk)
 values
('Alice in Wonderland', null);

select * from kniha;

/*
1;"Tajuplný ostrov";"Čeština"
2;"île mystérieuse";"Francouzština"
4;"Alice in Wonderland";""
*/

 -- =========
 -- 2. NULL 2
 -- =========
 
 -- V případě update, nebo delete nastavit hodnotu na null
-- Zruší se existující cizí klíč
ALTER TABLE public.kniha DROP CONSTRAINT kniha_jazyk_fkey;

-- Přidá se nový klíč u kterého se modifikuje akce na delete na CASCADE z NO ACTION
 ALTER TABLE public.kniha
  ADD CONSTRAINT kniha_jazyk_fkey FOREIGN KEY (jazyk)
      REFERENCES public.jazyk (jazyk) MATCH SIMPLE
      ON UPDATE SET NULL ON DELETE SET NULL;

-- Zrušíme záznam s tabulky jazyk "Francouzština"
delete from jazyk
  where jazyk = 'Francouzština';

-- Výsledek
select * from kniha where nazev = 'île mystérieuse';

/*
2;"île mystérieuse";""
*/

-- ===============
-- 3.  SET DEFAULT
-- ===============       

-- V případě, že nastavíme defualtní hodnotu při delete nebo update

insert into jazyk
values
('Neurčeno');

 -- V případě update, nebo delete nastavit hodnotu na null
-- Zruší se existující cizí klíč
ALTER TABLE public.kniha DROP CONSTRAINT kniha_jazyk_fkey;

-- Přidá se nový klíč u kterého se modifikuje akce na delete na CASCADE z NO ACTION
 ALTER TABLE public.kniha
  ADD CONSTRAINT kniha_jazyk_fkey FOREIGN KEY (jazyk)
      REFERENCES public.jazyk (jazyk) MATCH SIMPLE
      ON DELETE SET DEFAULT ;

-- Změní se defaultní hodnota pro sloupec

alter table kniha alter column jazyk set default 'Neurčeno'; 

-- Pak když vložíme nový záznam

insert into kniha
 (nazev)
 values    
('神秘岛');

-- Výběr
select * from kniha;

/*
1;"Tajuplný ostrov";"Čeština"
4;"Alice in Wonderland";""
2;"île mystérieuse";""
5;"神秘岛";"Neurčeno"

*/

-- Zrušení jazyka má za následek přidání defaultní hodnoty
delete from jazyk where jazyk != 'Neurčeno';

select * from kniha;
/*
4;"Alice in Wonderland";""
2;"île mystérieuse";""
5;"神秘岛";"Neurčeno"
1;"Tajuplný ostrov";"Neurčeno"
*/





